//---------------------------------------------------------------------------
//
//  Program:     MidiAndMusicXml.exe (Windows) MidiAndMusicXml.app (Mac)
//
//  File:        MidiAndMusicXmlPlayer.lpr
//
//  Copyright:   By Mogens Lundholm alias Magnus Lind
//               The program may be copied under these
//               conditions: 1. The style is not altered, indentions, name
//               style etc. To this copyright notice the change description
//               is added
//
//  Project:     MidiAnMusicXmlPlayer.dpr / MidiAnMusicXmlPlayer.lpi
//
//  Purpose:     To play a midi file format 0 or format 1 and Music XML-files
//               Possibility to
//               select tracks and speed and transpose the midi file
//
//  Compilation: Compile with Delphi 4 (MidiAnMusicXmlPlayer.dpr)
//               or Lazarus (MidiAnMusicXmlPlayer.lpi)
//               or Delphi Xe (MidiAnMusicXmlPlayer.dproj)
//
//  Description: From the midi file an array of events are build. The events
//               are only "short message", i.e. notes and control setting.
//               The meta events are skipped except data used for openoptions
//               printout (Name of song + track names.
//
//  Documents:   Score.ppt: Layout of a note page
//               XmlScorePartwise.odp: State diagram for XML, part 1
//               XmlPart.odp: State diagram for XML, part 2
//               Problems.txt: Not solved problems
//               Testplan.txt: How to test midi.exe
//
//  Version:     The About procedure uses the value VersionText defined
//               in UnitMidiDefinitions.pas
//---------------------------------------------------------------------------

program MidiAndMusicXmlPlayer;

{$MODE Delphi}

uses
{$ifdef UNIX}
      cthreads,
      cmem,
{$endif}

  Forms, Interfaces,
  UnitMessage in 'UnitMessage.pas' {FormMessage},
  UnitMidiGui in 'UnitMidi.pas' {FormMidi},
  UnitMidiIn in 'UnitMidiIn.pas' {FormMidiIn},
  UnitMidiMixer in 'UnitMidiMixer.pas' {FormMidiMixer},
  UnitMidiPlay in 'UnitMidiPlay.pas',
  UnitMidiDefinitions in 'UnitMidiDefinitions.pas',
  UnitText in 'UnitText.pas' {FormText},
  UnitManual in 'UnitManual.pas' {FormManual},
////  UnitOpenDialog in 'UnitOpenDialog.pas' {FormOpenDialog},
  UnitMidiOpenDialog in 'UnitMidiOpenDialog.pas' {FormMidiOpenDialog};

{$R *.res}

begin
  Errors:=0;
  Application.Initialize;
{$ifdef Darwin}
  if false then
{$else}
  if Paramcount>0 then
{$endif}
    begin
    Application.CreateForm(TFormMessage, FormMessage);
    Application.CreateForm(TFormMidiGui, FormMidiGui);
    end
  else
    begin
    Application.CreateForm(TFormMidiGui, FormMidiGui);
    Application.CreateForm(TFormMessage, FormMessage);
    end;

  FormMidiGui.Visible:=false;
  Application.CreateForm(TFormMidiIn, FormMidiIn);
  Application.CreateForm(TFormMidiMixer, FormMidiMixer);
  Application.CreateForm(TFormText, FormText);
  Application.CreateForm(TFormManual, FormManual);
////  Application.CreateForm(TFormOpenDialog, FormOpenDialog);
  Application.CreateForm(TFormMidiOpenDialog, FormMidiOpenDialog);
  Application.Run;
end.
